@@ -5,6 +5,8 @@ from typing import AsyncContextManager
 from async_timeout import asyncio
 from diskcache import Cache
 from threading import Lock
+
+import qiniu
 from hamunafs.utils.singleton_wrapper import Singleton
 from hamunafs.utils.redisutil import XRedis
 from hamunafs.utils.nsqmanager import MQManager
@@ -59,6 +61,13 @@ class Client(Singleton):
         
         self.update()
         self._inited = True
+
+    def get_qiniu_token(self, fname):
+        qiniu_backend_keys = [k for k in list(self.backend_pools.keys) if k.startswith('qiniu')]
+        if len(qiniu_backend_keys):
+            backend = random.random(qiniu_backend_keys)[0]
+            return backend.get_token(fname)
+        return None
     
     def update(self):
         with self.lock:
@@ -73,7 +82,7 @@ class Client(Singleton):
                     pool_data = resp['data']
                     for info in pool_data:
                         backend_pools[info['key']] = backend_factory[info['backend']](info['conf'])
-                        self.backend_pools = backend_pools
+                    self.backend_pools = backend_pools
                 else:
                     raise Exception('error on acquiring fs backends...')
             else:
@@ -91,10 +100,13 @@ class Client(Singleton):
             prefix, _url = url.split('://')
             
             if prefix in self.backend_pools:
-                bucket, bucket_name = _url.split('/')        
+                bucket, bucket_name = _url.split('/')
                 return self.backend_pools[prefix], bucket, bucket_name
             return None, None, None
 
+    def _get_backend(self, key):
+        return key, self.backend_pools[key]
+
     def cache(self, key, val, ttl=7200):
         if not isinstance(val, str):
             _val = json.dumps(val)
@@ -124,7 +136,8 @@ class Client(Singleton):
         cache_key = 'cloud_{}_{}'.format(bucket, bucket_name)
         cache_data = self.get_cache(cache_key, toObj=False)
         if cache_data is None:
-            prefix, backend = self._random_pick_backend()
+            # prefix, backend = self._random_pick_backend()
+            prefix, backend = self._get_backend('yao')
             
             ret, e = backend.put(path, bucket, bucket_name, tmp)
             if ret:
@@ -166,30 +179,34 @@ class Client(Singleton):
     def put_from_cloud(self, url, bucket, bucket_name, timeout=10, file_ttl=-1):
         # save to cache
         task_id = 'tmp_file_{}_{}'.format(bucket, bucket_name)
-        ret = self.mq.publish(self.put_topic, {
-            'url': url,
-            'bucket': bucket,
-            'bucket_name': bucket_name,
-            'ttl': file_ttl
-        })
-        
-        if ret:
-            t = time.time()
-            while True:
-                if time.time() - t >= timeout:
-                    break
-                
-                resp = self.redis.get(task_id)
-                
-                if resp is not None and len(resp) > 0:
-                    resp = json.loads(resp)
-                    if resp['ret']:
-                        return True, '{}/{}'.format(bucket, bucket_name)
-                    else:
-                        return False, resp['err']
-                
-                time.sleep(1)
-        return False, '上传出错'
+        try:
+            ret = self.mq.publish(self.put_topic, {
+                'url': url,
+                'bucket': bucket,
+                'bucket_name': bucket_name,
+                'ttl': file_ttl
+            })
+            
+            if ret:
+                t = time.time()
+                while True:
+                    if time.time() - t >= timeout:
+                        break
+                    
+                    resp = self.redis.get(task_id)
+                    
+                    if resp is not None and len(resp) > 0:
+                        resp = json.loads(resp)
+                        if resp['ret']:
+                            return True, '{}/{}'.format(bucket, bucket_name)
+                        else:
+                            return False, resp['err']
+                    
+                    time.sleep(1)
+            
+            return False, '上传出错'
+        except Exception as e:
+            return False, str(e)
     
     def put(self, path, bucket, bucket_name, timeout=10, file_ttl=-1):
         ret, e = self.put_to_cloud(path, bucket, bucket_name)
